@@ -1034,9 +1034,23 @@ namespace EventProcessing
         static void onTouchEvent( LocalEvent & eventHandler, const SDL_TouchFingerEvent & event )
         {
 #if defined( TARGET_PS_VITA )
-            if ( event.touchId != 0 ) {
-                // Ignore rear touchpad on PS Vita
-                return;
+            {
+                // PS Vita has two touchpads: front and rear. The ID of the front touchpad must match the value of
+                // 'SDL_TouchID' used in the 'SDL_AddTouch()' call in the 'VITA_InitTouch()' function in this SDL2
+                // source file: video/vita/SDL_vitatouch.c.
+                constexpr SDL_TouchID frontTouchpadDeviceID
+                {
+#if SDL_VERSION_ATLEAST( 2, 30, 7 )
+                    1
+#else
+                    0
+#endif
+                };
+
+                // Use only front touchpad on PS Vita.
+                if ( event.touchId != frontTouchpadDeviceID ) {
+                    return;
+                }
             }
 #endif
 
