@@ -26,6 +26,7 @@
 #include <cstddef>
 #include <cstdint>
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
@@ -81,11 +82,16 @@ namespace
         using Interface::ListBox<std::string>::ActionListSingleClick;
         using Interface::ListBox<std::string>::ActionListPressRight;
 
-        using ListBox::ListBox;
+        AnswerListBox( const fheroes2::Point & pt, const fheroes2::SupportedLanguage language )
+            : ListBox( pt )
+            , _language( language )
+        {
+            // Do nothing.
+        }
 
         void RedrawItem( const std::string & answer, int32_t posX, int32_t posY, bool current ) override
         {
-            fheroes2::Text text{ answer, ( current ? fheroes2::FontType::normalYellow() : fheroes2::FontType::normalWhite() ) };
+            fheroes2::Text text{ answer, ( current ? fheroes2::FontType::normalYellow() : fheroes2::FontType::normalWhite() ), _language };
             text.fitToOneRow( answerArea.width - 10 );
             text.draw( posX + 5, posY + 5, fheroes2::Display::instance() );
         }
@@ -107,7 +113,17 @@ namespace
 
         void ActionListDoubleClick( std::string & /*unused*/ ) override
         {
-            // Do nothing.
+            _isDoubleClicked = true;
+        }
+
+        bool isDoubleClicked() const
+        {
+            return _isDoubleClicked;
+        }
+
+        void resetDoubleClickedState()
+        {
+            _isDoubleClicked = false;
         }
 
         void ActionListSingleClick( std::string & /*unused*/ ) override
@@ -141,12 +157,16 @@ namespace
 
     private:
         std::unique_ptr<fheroes2::ImageRestorer> _listBackground;
+
+        bool _isDoubleClicked{ false };
+
+        const fheroes2::SupportedLanguage _language;
     };
 }
 
 namespace Editor
 {
-    bool openSphinxWindow( Maps::Map_Format::SphinxMetadata & metadata )
+    bool openSphinxWindow( Maps::Map_Format::SphinxMetadata & metadata, const fheroes2::SupportedLanguage language )
     {
         const CursorRestorer cursorRestorer( true, Cursor::POINTER );
 
@@ -171,7 +191,7 @@ namespace Editor
 
         text.draw( riddleRoi.x + ( riddleRoi.width - text.width() ) / 2, offsetY, display );
 
-        text.set( metadata.riddle, fheroes2::FontType::normalWhite() );
+        text.set( metadata.riddle, fheroes2::FontType::normalWhite(), language );
         text.draw( riddleRoi.x + 5, riddleRoi.y + 5, riddleRoi.width - 10, display );
 
         const fheroes2::Rect answerRoi{ windowArea.x + elementOffset + riddleRoi.width + elementOffset, offsetY + text.height(), answerArea.width, answerArea.height };
@@ -182,7 +202,7 @@ namespace Editor
 
         const bool isEvilInterface = Settings::Get().isEvilInterfaceEnabled();
 
-        AnswerListBox answerList( answerRoi.getPosition() );
+        AnswerListBox answerList( answerRoi.getPosition(), language );
         answerList.initListBackgroundRestorer( answerRoi );
 
         answerList.SetAreaItems( { answerRoi.x, answerRoi.y, answerRoi.width, answerRoi.height - listAreaHeightDeduction } );
@@ -204,17 +224,19 @@ namespace Editor
 
         answerList.Redraw();
 
-        const fheroes2::Sprite & buttonImage = fheroes2::AGG::GetICN( ICN::CELLWIN, 13 );
+        const int minibuttonIcnId = isEvilInterface ? ICN::CELLWIN_EVIL : ICN::CELLWIN;
+
+        const fheroes2::Sprite & buttonImage = fheroes2::AGG::GetICN( minibuttonIcnId, 13 );
         const int32_t buttonWidth = buttonImage.width();
         const int32_t buttonOffset = ( answerArea.width - 3 * buttonWidth ) / 2 + buttonWidth;
 
-        fheroes2::Button buttonAdd( answerRoi.x, answerRoi.y + answerRoi.height + 5, ICN::CELLWIN, 13, 14 );
+        fheroes2::Button buttonAdd( answerRoi.x, answerRoi.y + answerRoi.height + 5, minibuttonIcnId, 13, 14 );
         buttonAdd.draw();
 
-        fheroes2::Button buttonEdit( answerRoi.x + buttonOffset, answerRoi.y + answerRoi.height + 5, ICN::CELLWIN, 15, 16 );
+        fheroes2::Button buttonEdit( answerRoi.x + buttonOffset, answerRoi.y + answerRoi.height + 5, minibuttonIcnId, 15, 16 );
         buttonEdit.draw();
 
-        fheroes2::Button buttonDelete( answerRoi.x + answerArea.width - buttonWidth, answerRoi.y + answerRoi.height + 5, ICN::CELLWIN, 17, 18 );
+        fheroes2::Button buttonDelete( answerRoi.x + answerArea.width - buttonWidth, answerRoi.y + answerRoi.height + 5, minibuttonIcnId, 17, 18 );
         buttonDelete.draw();
 
         offsetY += text.height() + riddleArea.height + elementOffset;
@@ -235,7 +257,7 @@ namespace Editor
 
         redrawArtifactImage( metadata.artifact );
 
-        fheroes2::Button buttonDeleteArtifact( artifactRoi.x + ( artifactRoi.width - buttonWidth ) / 2, artifactRoi.y + artifactRoi.height + 5, ICN::CELLWIN, 17, 18 );
+        fheroes2::Button buttonDeleteArtifact( artifactRoi.x + ( artifactRoi.width - buttonWidth ) / 2, artifactRoi.y + artifactRoi.height + 5, minibuttonIcnId, 17, 18 );
         buttonDeleteArtifact.draw();
 
         const fheroes2::Rect resourceRoi{ answerRoi.x, offsetY + text.height(), answerRoi.width, 99 };
@@ -258,12 +280,12 @@ namespace Editor
 
         LocalEvent & le = LocalEvent::Get();
         while ( le.HandleEvents() ) {
-            buttonOk.drawOnState( le.MousePressLeft( buttonOk.area() ) );
-            buttonCancel.drawOnState( le.MousePressLeft( buttonCancel.area() ) );
-            buttonAdd.drawOnState( le.MousePressLeft( buttonAdd.area() ) );
-            buttonEdit.drawOnState( le.MousePressLeft( buttonEdit.area() ) );
-            buttonDelete.drawOnState( le.MousePressLeft( buttonDelete.area() ) );
-            buttonDeleteArtifact.drawOnState( le.MousePressLeft( buttonDeleteArtifact.area() ) );
+            buttonOk.drawOnState( le.isMouseLeftButtonPressedInArea( buttonOk.area() ) );
+            buttonCancel.drawOnState( le.isMouseLeftButtonPressedInArea( buttonCancel.area() ) );
+            buttonAdd.drawOnState( le.isMouseLeftButtonPressedInArea( buttonAdd.area() ) );
+            buttonEdit.drawOnState( le.isMouseLeftButtonPressedInArea( buttonEdit.area() ) );
+            buttonDelete.drawOnState( le.isMouseLeftButtonPressedInArea( buttonDelete.area() ) );
+            buttonDeleteArtifact.drawOnState( le.isMouseLeftButtonPressedInArea( buttonDeleteArtifact.area() ) );
 
             if ( le.MouseClickLeft( buttonCancel.area() ) || Game::HotKeyPressEvent( Game::HotKeyEvent::DEFAULT_CANCEL ) ) {
                 break;
@@ -286,10 +308,10 @@ namespace Editor
                     int32_t * resourcePtr = metadata.resources.GetPtr( resourceType );
                     assert( resourcePtr != nullptr );
 
-                    uint32_t temp = *resourcePtr;
+                    int32_t temp = *resourcePtr;
 
                     if ( Dialog::SelectCount( Resource::String( resourceType ), 0, 1000000, temp, 1 ) ) {
-                        *resourcePtr = static_cast<int32_t>( temp );
+                        *resourcePtr = temp;
                     }
 
                     resourceRoiRestorer.restore();
@@ -303,18 +325,20 @@ namespace Editor
             if ( le.MouseClickLeft( riddleRoi ) ) {
                 std::string temp = metadata.riddle;
 
-                if ( Dialog::inputString( _( "Riddle:" ), temp, {}, longestRiddle, true, true ) ) {
+                const fheroes2::Text body{ _( "Riddle:" ), fheroes2::FontType::normalWhite() };
+                if ( Dialog::inputString( fheroes2::Text{}, body, temp, longestRiddle, true, language ) ) {
                     metadata.riddle = std::move( temp );
 
                     riddleRoiRestorer.restore();
-                    text.set( metadata.riddle, fheroes2::FontType::normalWhite() );
+                    text.set( metadata.riddle, fheroes2::FontType::normalWhite(), language );
                     text.draw( riddleRoi.x + 5, riddleRoi.y + 5, riddleRoi.width - 10, display );
                     isRedrawNeeded = true;
                 }
             }
             else if ( le.MouseClickLeft( buttonAdd.area() ) ) {
                 std::string newAnswer;
-                if ( Dialog::inputString( _( "Answer:" ), newAnswer, {}, longestAnswer, false, true ) ) {
+                const fheroes2::Text body{ _( "Answer:" ), fheroes2::FontType::normalWhite() };
+                if ( Dialog::inputString( fheroes2::Text{}, body, newAnswer, longestAnswer, false, language ) ) {
                     if ( std::any_of( metadata.answers.begin(), metadata.answers.end(), [&newAnswer]( const auto & answer ) { return answer == newAnswer; } ) ) {
                         fheroes2::showStandardTextMessage( _( "Answer" ), _( "This answer exists in the list." ), Dialog::OK );
                         continue;
@@ -327,13 +351,17 @@ namespace Editor
                     isRedrawNeeded = true;
                 }
             }
-            else if ( le.MouseClickLeft( buttonEdit.area() ) ) {
+            else if ( answerList.isDoubleClicked() || le.MouseClickLeft( buttonEdit.area() ) ) {
                 if ( answerList.getCurrentId() < 0 ) {
                     continue;
                 }
 
+                answerList.resetDoubleClickedState();
+
                 std::string temp = answerList.GetCurrent();
-                if ( Dialog::inputString( _( "Answer:" ), temp, {}, longestAnswer, false, true ) ) {
+
+                const fheroes2::Text body{ _( "Answer:" ), fheroes2::FontType::normalWhite() };
+                if ( Dialog::inputString( fheroes2::Text{}, body, temp, longestAnswer, false, language ) ) {
                     const auto count = std::count_if( metadata.answers.begin(), metadata.answers.end(), [&temp]( const auto & answer ) { return answer == temp; } );
                     if ( answerList.GetCurrent() != temp && count > 0 ) {
                         fheroes2::showStandardTextMessage( _( "Answer" ), _( "This answer exists in the list." ), Dialog::OK );
@@ -357,7 +385,7 @@ namespace Editor
                 isRedrawNeeded = true;
             }
             else if ( le.MouseClickLeft( artifactRoi ) ) {
-                const Artifact artifact = Dialog::selectArtifact( metadata.artifact );
+                const Artifact artifact = Dialog::selectArtifact( metadata.artifact, false );
                 if ( artifact.isValid() ) {
                     int32_t artifactMetadata = metadata.artifactMetadata;
 
@@ -393,42 +421,40 @@ namespace Editor
 
                 display.render( artifactRoi );
             }
-            else if ( le.MousePressRight( buttonCancel.area() ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( buttonCancel.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Cancel" ), _( "Exit this menu without doing anything." ), Dialog::ZERO );
             }
-            else if ( le.MousePressRight( buttonOk.area() ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( buttonOk.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Okay" ), _( "Click to save the Sphinx properties." ), Dialog::ZERO );
             }
-            else if ( le.MousePressRight( buttonAdd.area() ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( buttonAdd.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Add Answer" ), _( "Add an additional answer for the question." ), Dialog::ZERO );
             }
-            else if ( le.MousePressRight( buttonEdit.area() ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( buttonEdit.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Edit Answer" ), _( "Edit an existing answer for the question." ), Dialog::ZERO );
             }
-            else if ( le.MousePressRight( buttonDelete.area() ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( buttonDelete.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Delete Answer" ), _( "Delete an existing answer for the question." ), Dialog::ZERO );
             }
-            else if ( le.MousePressRight( artifactRoi ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( artifactRoi ) ) {
                 // Since Artifact class does not allow to set a random spell (for obvious reasons),
                 // we have to use special UI code to render the popup window with all needed information.
                 const Artifact artifact( metadata.artifact );
 
                 if ( artifact.isValid() ) {
-                    const fheroes2::Text header( artifact.GetName(), fheroes2::FontType::normalYellow() );
-                    const fheroes2::Text description( fheroes2::getArtifactData( metadata.artifact ).getDescription( metadata.artifactMetadata ),
-                                                      fheroes2::FontType::normalWhite() );
-
                     fheroes2::ArtifactDialogElement artifactUI( artifact );
-                    fheroes2::showMessage( header, description, Dialog::ZERO, { &artifactUI } );
+
+                    fheroes2::showStandardTextMessage( artifact.GetName(), fheroes2::getArtifactData( metadata.artifact ).getDescription( metadata.artifactMetadata ),
+                                                       Dialog::ZERO, { &artifactUI } );
                 }
                 else {
                     fheroes2::showStandardTextMessage( _( "Artifact" ), _( "No artifact will be given as a reward." ), Dialog::ZERO );
                 }
             }
-            else if ( le.MousePressRight( buttonDeleteArtifact.area() ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( buttonDeleteArtifact.area() ) ) {
                 fheroes2::showStandardTextMessage( _( "Delete Artifact" ), _( "Delete an artifact from the reward." ), Dialog::ZERO );
             }
-            else if ( le.MousePressRight( resourceRoi ) ) {
+            else if ( le.isMouseRightButtonPressedInArea( resourceRoi ) ) {
                 if ( metadata.resources.GetValidItemsCount() == 0 ) {
                     fheroes2::showStandardTextMessage( _( "Resources" ), _( "No resources will be given as a reward." ), Dialog::ZERO );
                 }
