@@ -225,6 +225,22 @@ public:
         return id == COLDRAY || id == COLDRING;
     }
 
+    bool isElementalSpell() const
+    {
+        switch ( id ) {
+        case COLDRAY:
+        case COLDRING:
+        case FIREBALL:
+        case FIREBLAST:
+        case LIGHTNINGBOLT:
+        case CHAINLIGHTNING:
+        case ELEMENTALSTORM:
+            return true;
+        default:
+            return false;
+        }
+    }
+
     bool isBuiltinOnly() const
     {
         return id == PETRIFY;
@@ -235,9 +251,7 @@ public:
     // Returns the index of the spell sprite in SPELLS.ICN
     uint32_t IndexSprite() const;
 
-    static Spell Rand( const int level, const bool isAdventure );
-    static Spell RandCombat( const int level );
-    static Spell RandAdventure( const int level );
+    static Spell getRandomSpell( const int level );
 
     // Returns the IDs of all spells of a given level that are suitable for the spell book (i.e. no placeholders or exclusive
     // built-in spells for monsters are returned). If 'spellLevel' is less than 1, suitable spells of all levels are returned.
