@@ -27,14 +27,11 @@
 #include <array>
 #include <cassert>
 #include <limits>
-#include <memory>
 #include <ostream>
 #include <set>
 #include <tuple>
-#include <type_traits>
-#include <utility>
 
-#include "ai.h"
+#include "ai_planner.h"
 #include "artifact.h"
 #include "campaign_savedata.h"
 #include "campaign_scenariodata.h"
@@ -44,7 +41,6 @@
 #include "game.h"
 #include "game_io.h"
 #include "game_over.h"
-#include "gamedefs.h"
 #include "ground.h"
 #include "heroes.h"
 #include "logging.h"
@@ -68,7 +64,7 @@
 
 namespace
 {
-    bool isTileBlockedForSettingMonster( const std::vector<Maps::Tiles> & mapTiles, const int32_t tileId, const int32_t radius, const std::set<int32_t> & excludeTiles )
+    bool isTileBlockedForSettingMonster( const std::vector<Maps::Tile> & mapTiles, const int32_t tileId, const int32_t radius, const std::set<int32_t> & excludeTiles )
     {
         const MapsIndexes & indexes = Maps::getAroundIndexes( tileId, radius );
 
@@ -77,12 +73,12 @@ namespace
                 return true;
             }
 
-            const Maps::Tiles & indexedTile = mapTiles[indexId];
+            const Maps::Tile & indexedTile = mapTiles[indexId];
             if ( indexedTile.isWater() ) {
                 continue;
             }
 
-            const MP2::MapObjectType objectType = indexedTile.GetObject( true );
+            const MP2::MapObjectType objectType = indexedTile.getMainObjectType( true );
             if ( objectType == MP2::OBJ_CASTLE || objectType == MP2::OBJ_HERO || objectType == MP2::OBJ_MONSTER ) {
                 return true;
             }
@@ -91,7 +87,7 @@ namespace
         return false;
     }
 
-    int32_t findSuitableNeighbouringTile( const std::vector<Maps::Tiles> & mapTiles, const int32_t tileId, const bool allDirections )
+    int32_t findSuitableNeighbouringTile( const std::vector<Maps::Tile> & mapTiles, const int32_t tileId, const bool allDirections )
     {
         std::vector<int32_t> suitableIds;
 
@@ -103,14 +99,14 @@ namespace
                 continue;
             }
 
-            const Maps::Tiles & indexedTile = mapTiles[indexId];
+            const Maps::Tile & indexedTile = mapTiles[indexId];
 
             if ( indexedTile.isWater() || !isClearGround( indexedTile ) ) {
                 continue;
             }
 
             // If the candidate tile is a coast tile, it is suitable only if there are other coast tiles nearby
-            if ( indexedTile.GetObject( false ) == MP2::OBJ_COAST ) {
+            if ( indexedTile.getMainObjectType( false ) == MP2::OBJ_COAST ) {
                 const MapsIndexes coastTiles = Maps::ScanAroundObject( indexId, MP2::OBJ_COAST );
 
                 if ( coastTiles.empty() ) {
@@ -128,14 +124,14 @@ namespace
         return Rand::Get( suitableIds );
     }
 
-    int32_t getNeighbouringEmptyTileCount( const std::vector<Maps::Tiles> & mapTiles, const int32_t tileId )
+    int32_t getNeighbouringEmptyTileCount( const std::vector<Maps::Tile> & mapTiles, const int32_t tileId )
     {
         int32_t count = 0;
 
         const MapsIndexes & indexes = Maps::getAroundIndexes( tileId );
 
         for ( const int32_t indexId : indexes ) {
-            const Maps::Tiles & indexedTile = mapTiles[indexId];
+            const Maps::Tile & indexedTile = mapTiles[indexId];
             if ( indexedTile.isWater() || !isClearGround( indexedTile ) ) {
                 continue;
             }
@@ -147,81 +143,70 @@ namespace
     }
 }
 
-MapObjects::~MapObjects()
+MapObjectSimple * MapObjects::get( const uint32_t uid ) const
 {
-    clear();
-}
-
-void MapObjects::clear()
-{
-    for ( iterator it = begin(); it != end(); ++it )
-        delete ( *it ).second;
-    std::map<uint32_t, MapObjectSimple *>::clear();
-}
-
-void MapObjects::add( MapObjectSimple * obj )
-{
-    if ( obj ) {
-        std::map<uint32_t, MapObjectSimple *> & currentMap = *this;
-        if ( currentMap[obj->GetUID()] )
-            delete currentMap[obj->GetUID()];
-        currentMap[obj->GetUID()] = obj;
+    if ( const auto iter = _objects.find( uid ); iter != _objects.end() ) {
+        return iter->second.get();
     }
+
+    return nullptr;
 }
 
-MapObjectSimple * MapObjects::get( uint32_t uid )
+std::list<MapObjectSimple *> MapObjects::get( const fheroes2::Point & pos ) const
 {
-    iterator it = find( uid );
-    return it != end() ? ( *it ).second : nullptr;
+    std::list<MapObjectSimple *> result;
+
+    for ( const auto & [dummy, obj] : _objects ) {
+        assert( obj );
+
+        if ( !obj->isPosition( pos ) ) {
+            continue;
+        }
+
+        result.push_back( obj.get() );
+    }
+
+    return result;
 }
 
-std::list<MapObjectSimple *> MapObjects::get( const fheroes2::Point & pos )
+void MapObjects::remove( const uint32_t uid )
 {
-    std::list<MapObjectSimple *> res;
-    for ( iterator it = begin(); it != end(); ++it )
-        if ( ( *it ).second && ( *it ).second->isPosition( pos ) )
-            res.push_back( ( *it ).second );
-    return res;
+    _objects.erase( uid );
 }
 
-void MapObjects::remove( uint32_t uid )
+CapturedObject & CapturedObjects::Get( const int32_t index )
 {
-    iterator it = find( uid );
-    if ( it != end() )
-        delete ( *it ).second;
-    erase( it );
+    return operator[]( index );
 }
 
-CapturedObject & CapturedObjects::Get( int32_t index )
-{
-    std::map<int32_t, CapturedObject> & my = *this;
-    return my[index];
-}
-
-void CapturedObjects::SetColor( int32_t index, int col )
+void CapturedObjects::SetColor( const int32_t index, const int col )
 {
     Get( index ).SetColor( col );
 }
 
-void CapturedObjects::Set( int32_t index, int obj, int col )
+void CapturedObjects::Set( const int32_t index, const MP2::MapObjectType obj, const int col )
 {
-    CapturedObject & co = Get( index );
+    CapturedObject & capturedObj = Get( index );
 
-    if ( co.GetColor() != col && co.guardians.isValid() )
-        co.guardians.Reset();
+    if ( capturedObj.GetColor() != col && capturedObj.guardians.isValid() ) {
+        capturedObj.guardians.Reset();
+    }
 
-    co.Set( obj, col );
+    capturedObj.Set( obj, col );
 }
 
-uint32_t CapturedObjects::GetCount( int obj, int col ) const
+uint32_t CapturedObjects::GetCount( const MP2::MapObjectType obj, const int col ) const
 {
     uint32_t result = 0;
 
-    const ObjectColor objcol( obj, col );
+    const ObjectColor objCol( obj, col );
 
-    for ( const_iterator it = begin(); it != end(); ++it ) {
-        if ( objcol == ( *it ).second.objcol )
-            ++result;
+    for ( const auto & [idx, capturedObj] : *this ) {
+        if ( capturedObj.objCol != objCol ) {
+            continue;
+        }
+
+        ++result;
     }
 
     return result;
@@ -230,63 +215,77 @@ uint32_t CapturedObjects::GetCount( int obj, int col ) const
 uint32_t CapturedObjects::GetCountMines( const int resourceType, const int ownerColor ) const
 {
     uint32_t count = 0;
-    const ObjectColor correctObject( MP2::OBJ_MINE, ownerColor );
 
-    for ( const auto & [tileIndex, objectInfo] : *this ) {
-        if ( correctObject == objectInfo.objcol ) {
-            const int32_t mineResource = Maps::getDailyIncomeObjectResources( world.GetTiles( tileIndex ) ).getFirstValidResource().first;
-            if ( resourceType == mineResource ) {
-                ++count;
-            }
+    const ObjectColor objCol( MP2::OBJ_MINE, ownerColor );
+
+    for ( const auto & [idx, capturedObj] : *this ) {
+        if ( capturedObj.objCol != objCol ) {
+            continue;
         }
+
+        if ( resourceType != Maps::getDailyIncomeObjectResources( world.getTile( idx ) ).getFirstValidResource().first ) {
+            continue;
+        }
+
+        ++count;
     }
 
     return count;
 }
 
-int CapturedObjects::GetColor( int32_t index ) const
+int CapturedObjects::GetColor( const int32_t index ) const
 {
-    const_iterator it = find( index );
-    return it != end() ? ( *it ).second.GetColor() : Color::NONE;
+    const auto iter = find( index );
+    if ( iter == end() ) {
+        return Color::NONE;
+    }
+
+    return iter->second.GetColor();
 }
 
-void CapturedObjects::ClearFog( int colors )
+void CapturedObjects::ClearFog( const int colors ) const
 {
-    // clear abroad objects
-    for ( const_iterator it = begin(); it != end(); ++it ) {
-        const ObjectColor & objcol = ( *it ).second.objcol;
+    for ( const auto & [idx, capturedObj] : *this ) {
+        const ObjectColor & objCol = capturedObj.objCol;
 
-        if ( objcol.isColor( colors ) ) {
-            int scoutingDistance = 0;
-
-            switch ( objcol.first ) {
-            case MP2::OBJ_MINE:
-            case MP2::OBJ_ALCHEMIST_LAB:
-            case MP2::OBJ_SAWMILL:
-                scoutingDistance = 2;
-                break;
-
-            default:
-                break;
-            }
-
-            if ( scoutingDistance )
-                Maps::ClearFog( ( *it ).first, scoutingDistance, colors );
+        if ( !objCol.isColor( colors ) ) {
+            continue;
         }
+
+        int scoutingDistance = 0;
+
+        switch ( objCol.first ) {
+        case MP2::OBJ_MINE:
+        case MP2::OBJ_ALCHEMIST_LAB:
+        case MP2::OBJ_SAWMILL:
+            scoutingDistance = 2;
+            break;
+
+        default:
+            break;
+        }
+
+        if ( scoutingDistance == 0 ) {
+            continue;
+        }
+
+        Maps::ClearFog( idx, scoutingDistance, colors );
     }
 }
 
-void CapturedObjects::ResetColor( int color )
+void CapturedObjects::ResetColor( const int color )
 {
-    for ( iterator it = begin(); it != end(); ++it ) {
-        ObjectColor & objcol = ( *it ).second.objcol;
+    for ( auto & [idx, capturedObj] : *this ) {
+        ObjectColor & objCol = capturedObj.objCol;
 
-        if ( objcol.isColor( color ) ) {
-            const MP2::MapObjectType objectType = static_cast<MP2::MapObjectType>( objcol.first );
-
-            objcol.second = objectType == MP2::OBJ_CASTLE ? Color::UNUSED : Color::NONE;
-            world.GetTiles( it->first ).setOwnershipFlag( objectType, objcol.second );
+        if ( !objCol.isColor( color ) ) {
+            continue;
         }
+
+        auto & [objectType, col] = objCol;
+
+        col = Color::NONE;
+        world.getTile( idx ).setOwnershipFlag( objectType, col );
     }
 }
 
@@ -335,7 +334,7 @@ void World::Reset()
     vec_castles.Clear();
 
     // heroes
-    vec_heroes.clear();
+    vec_heroes.Clear();
 
     // extra
     map_captureobj.clear();
@@ -374,7 +373,7 @@ void World::generateBattleOnlyMap()
         fi.version = GameVersion::PRICE_OF_LOYALTY;
     }
 
-    conf.SetCurrentFileInfo( std::move( fi ) );
+    conf.setCurrentMapInfo( std::move( fi ) );
 
     Defaults();
 
@@ -409,7 +408,7 @@ void World::generateForEditor( const int32_t size )
 
     fi.version = GameVersion::PRICE_OF_LOYALTY;
 
-    conf.SetCurrentFileInfo( std::move( fi ) );
+    conf.setCurrentMapInfo( std::move( fi ) );
 
     Defaults();
 
@@ -446,7 +445,7 @@ Castle * World::getCastleEntrance( const fheroes2::Point & tilePosition )
 
 bool World::isValidCastleEntrance( const fheroes2::Point & tilePosition ) const
 {
-    return Maps::isValidAbsPoint( tilePosition.x, tilePosition.y ) && ( GetTiles( tilePosition.x, tilePosition.y ).GetObject( false ) == MP2::OBJ_CASTLE );
+    return Maps::isValidAbsPoint( tilePosition.x, tilePosition.y ) && ( getTile( tilePosition.x, tilePosition.y ).getMainObjectType( false ) == MP2::OBJ_CASTLE );
 }
 
 Heroes * World::GetHeroForHire( const int race, const int heroIDToIgnore /* = Heroes::UNKNOWN */ ) const
@@ -466,37 +465,37 @@ Heroes * World::GetHero( const Castle & castle ) const
 
 int World::GetDay() const
 {
-    return LastDay() ? DAYOFWEEK : day % DAYOFWEEK;
+    return LastDay() ? numOfDaysPerWeek : day % numOfDaysPerWeek;
 }
 
 int World::GetWeek() const
 {
-    return LastWeek() ? WEEKOFMONTH : week % WEEKOFMONTH;
+    return LastWeek() ? numOfWeeksPerMonth : week % numOfWeeksPerMonth;
 }
 
 bool World::BeginWeek() const
 {
-    return 1 == ( day % DAYOFWEEK );
+    return 1 == ( day % numOfDaysPerWeek );
 }
 
 bool World::BeginMonth() const
 {
-    return 1 == ( week % WEEKOFMONTH ) && BeginWeek();
+    return 1 == ( week % numOfWeeksPerMonth ) && BeginWeek();
 }
 
 bool World::LastDay() const
 {
-    return ( 0 == ( day % DAYOFWEEK ) );
+    return ( 0 == ( day % numOfDaysPerWeek ) );
 }
 
 bool World::FirstWeek() const
 {
-    return ( 1 == ( week % WEEKOFMONTH ) );
+    return ( 1 == ( week % numOfWeeksPerMonth ) );
 }
 
 bool World::LastWeek() const
 {
-    return ( 0 == ( week % WEEKOFMONTH ) );
+    return ( 0 == ( week % numOfWeeksPerMonth ) );
 }
 
 const Week & World::GetWeekType() const
@@ -559,8 +558,8 @@ void World::NewWeek()
 {
     // update objects
     if ( week > 1 ) {
-        for ( Maps::Tiles & tile : vec_tiles ) {
-            if ( MP2::isWeekLife( tile.GetObject( false ) ) || tile.GetObject() == MP2::OBJ_MONSTER ) {
+        for ( Maps::Tile & tile : vec_tiles ) {
+            if ( MP2::isWeekLife( tile.getMainObjectType( false ) ) || tile.getMainObjectType() == MP2::OBJ_MONSTER ) {
                 updateObjectInfoTile( tile, false );
             }
         }
@@ -604,14 +603,14 @@ void World::MonthOfMonstersAction( const Monster & mons )
 
     std::set<int32_t> excludeTiles;
 
-    for ( const Maps::Tiles & tile : vec_tiles ) {
+    for ( const Maps::Tile & tile : vec_tiles ) {
         if ( tile.isWater() ) {
             // Monsters are not placed on water.
             continue;
         }
 
         const int32_t tileId = tile.GetIndex();
-        const MP2::MapObjectType objectType = tile.GetObject( true );
+        const MP2::MapObjectType objectType = tile.getMainObjectType( true );
 
         if ( objectType == MP2::OBJ_CASTLE || objectType == MP2::OBJ_HERO || objectType == MP2::OBJ_MONSTER ) {
             excludeTiles.emplace( tileId );
@@ -708,7 +707,7 @@ void World::MonthOfMonstersAction( const Monster & mons )
     }
 }
 
-std::string World::getCurrentRumor() const
+std::pair<std::string, std::optional<fheroes2::SupportedLanguage>> World::getCurrentRumor() const
 {
     const uint32_t standardRumorCount = 10;
     const uint32_t totalRumorCount = static_cast<uint32_t>( _customRumors.size() ) + standardRumorCount;
@@ -718,7 +717,7 @@ std::string World::getCurrentRumor() const
     case 0: {
         std::string rumor( _( "The ultimate artifact is really the %{name}." ) );
         StringReplace( rumor, "%{name}", ultimate_artifact.GetName() );
-        return rumor;
+        return { rumor, std::nullopt };
     }
     case 1: {
         std::string rumor( _( "The ultimate artifact may be found in the %{name} regions of the world." ) );
@@ -756,50 +755,59 @@ std::string World::getCurrentRumor() const
         else {
             StringReplace( rumor, "%{name}", _( "south-east" ) );
         }
-        return rumor;
+        return { rumor, std::nullopt };
     }
     case 2:
-        return _( "The truth is out there." );
+        return { _( "The truth is out there." ), std::nullopt };
     case 3:
-        return _( "The dark side is stronger." );
+        return { _( "The dark side is stronger." ), std::nullopt };
     case 4:
-        return _( "The end of the world is near." );
+        return { _( "The end of the world is near." ), std::nullopt };
     case 5:
-        return _( "The bones of Lord Slayer are buried in the foundation of the arena." );
+        return { _( "The bones of Lord Slayer are buried in the foundation of the arena." ), std::nullopt };
     case 6:
-        return _( "A Black Dragon will take out a Titan any day of the week." );
+        return { _( "A Black Dragon will take out a Titan any day of the week." ), std::nullopt };
     case 7:
-        return _( "He told her: Yada yada yada... and then she said: Blah, blah, blah..." );
+        return { _( "He told her: Yada yada yada... and then she said: Blah, blah, blah..." ), std::nullopt };
     case 8:
-        return _( "An unknown force is being resurrected..." );
+        return { _( "An unknown force is being resurrected..." ), std::nullopt };
     case 9:
-        return _( "Check the newest version of the game at\nhttps://github.com/ihhub/\nfheroes2/releases" );
+        return { _( "Check the newest version of the game at\nhttps://github.com/ihhub/\nfheroes2/releases" ), std::nullopt };
     default:
         break;
     }
 
     assert( chosenRumorId >= standardRumorCount && chosenRumorId < totalRumorCount );
-    return _customRumors[chosenRumorId - standardRumorCount];
+    return { _customRumors[chosenRumorId - standardRumorCount], Settings::Get().getCurrentMapInfo().getSupportedLanguage() };
 }
 
 MapsIndexes World::GetTeleportEndPoints( const int32_t index ) const
 {
     MapsIndexes result;
 
-    const Maps::Tiles & entranceTile = GetTiles( index );
+    const Maps::Tile & entranceTile = getTile( index );
 
-    if ( entranceTile.GetObject( false ) != MP2::OBJ_STONE_LITHS ) {
+    if ( entranceTile.getMainObjectType( false ) != MP2::OBJ_STONE_LITHS ) {
+        return result;
+    }
+
+    const Maps::ObjectPart * entranceObjectPart = Maps::getObjectPartByActionType( entranceTile, MP2::OBJ_STONE_LITHS );
+    if ( entranceObjectPart == nullptr ) {
+        // This tile is marked as Stone Liths but somehow it doesn't even have Stone Liths' object parts.
+        assert( 0 );
         return result;
     }
 
     // The type of destination stone liths must match the type of the source stone liths.
-    for ( const int32_t teleportIndex : _allTeleports.at( entranceTile.GetObjectSpriteIndex() ) ) {
-        const Maps::Tiles & teleportTile = GetTiles( teleportIndex );
+    for ( const int32_t teleportIndex : _allTeleports.at( entranceObjectPart->icnIndex ) ) {
+        const Maps::Tile & teleportTile = getTile( teleportIndex );
 
-        if ( teleportIndex == index || teleportTile.getHero() != nullptr || teleportTile.isWater() != entranceTile.isWater() ) {
+        if ( teleportIndex == index || teleportTile.getMainObjectType() != MP2::OBJ_STONE_LITHS || teleportTile.isWater() != entranceTile.isWater() ) {
             continue;
         }
 
+        assert( teleportTile.getHero() == nullptr );
+
         result.push_back( teleportIndex );
     }
 
@@ -821,20 +829,38 @@ MapsIndexes World::GetWhirlpoolEndPoints( const int32_t index ) const
 {
     MapsIndexes result;
 
-    const Maps::Tiles & entranceTile = GetTiles( index );
+    const Maps::Tile & entranceTile = getTile( index );
 
-    if ( entranceTile.GetObject( false ) != MP2::OBJ_WHIRLPOOL ) {
+    if ( entranceTile.getMainObjectType( false ) != MP2::OBJ_WHIRLPOOL ) {
         return result;
     }
 
-    // The exit point from the destination whirlpool must match the entry point in the source whirlpool.
-    for ( const int32_t whirlpoolIndex : _allWhirlpools.at( entranceTile.GetObjectSpriteIndex() ) ) {
-        const Maps::Tiles & whirlpoolTile = GetTiles( whirlpoolIndex );
+    const Maps::ObjectPart * entranceObjectPart = Maps::getObjectPartByActionType( entranceTile, MP2::OBJ_WHIRLPOOL );
+    if ( entranceObjectPart == nullptr ) {
+        // This tile is marked as Whirlpool but somehow it doesn't even have whirlpool's object parts.
+        assert( 0 );
+        return result;
+    }
 
-        if ( whirlpoolTile.GetObjectUID() == entranceTile.GetObjectUID() || whirlpoolTile.getHero() != nullptr ) {
+    for ( const int32_t whirlpoolIndex : _allWhirlpools.at( entranceObjectPart->icnIndex ) ) {
+        const Maps::Tile & whirlpoolTile = getTile( whirlpoolIndex );
+        if ( whirlpoolTile.getMainObjectType() != MP2::OBJ_WHIRLPOOL ) {
             continue;
         }
 
+        const Maps::ObjectPart * destinationObjectPart = Maps::getObjectPartByActionType( whirlpoolTile, MP2::OBJ_WHIRLPOOL );
+        if ( destinationObjectPart == nullptr ) {
+            // This tile is marked as Whirlpool but somehow it doesn't even have whirlpool's object parts.
+            assert( 0 );
+            continue;
+        }
+
+        if ( destinationObjectPart->_uid == entranceObjectPart->_uid ) {
+            continue;
+        }
+
+        assert( whirlpoolTile.getHero() == nullptr );
+
         result.push_back( whirlpoolIndex );
     }
 
@@ -852,12 +878,12 @@ int32_t World::NextWhirlpool( const int32_t index ) const
     return Rand::Get( whilrpools );
 }
 
-uint32_t World::CountCapturedObject( int obj, int col ) const
+uint32_t World::CountCapturedObject( const MP2::MapObjectType obj, const int color ) const
 {
-    return map_captureobj.GetCount( obj, col );
+    return map_captureobj.GetCount( obj, color );
 }
 
-uint32_t World::CountCapturedMines( int type, int color ) const
+uint32_t World::CountCapturedMines( const int type, const int color ) const
 {
     switch ( type ) {
     case Resource::WOOD:
@@ -871,37 +897,41 @@ uint32_t World::CountCapturedMines( int type, int color ) const
     return map_captureobj.GetCountMines( type, color );
 }
 
-void World::CaptureObject( int32_t index, int color )
+void World::CaptureObject( const int32_t index, const int color )
 {
-    const MP2::MapObjectType objectType = GetTiles( index ).GetObject( false );
+    assert( CountBits( color ) <= 1 );
+
+    const MP2::MapObjectType objectType = getTile( index ).getMainObjectType( false );
     map_captureobj.Set( index, objectType, color );
 
+    if ( color != Color::NONE && !( color & Color::ALL ) ) {
+        return;
+    }
+
     Castle * castle = getCastleEntrance( Maps::GetPoint( index ) );
     if ( castle && castle->GetColor() != color ) {
         castle->ChangeColor( color );
     }
 
-    if ( color & ( Color::ALL | Color::UNUSED ) ) {
-        GetTiles( index ).setOwnershipFlag( objectType, color );
-    }
+    getTile( index ).setOwnershipFlag( objectType, color );
 }
 
-int World::ColorCapturedObject( int32_t index ) const
+int World::ColorCapturedObject( const int32_t index ) const
 {
     return map_captureobj.GetColor( index );
 }
 
-CapturedObject & World::GetCapturedObject( int32_t index )
+CapturedObject & World::GetCapturedObject( const int32_t index )
 {
     return map_captureobj.Get( index );
 }
 
-void World::ResetCapturedObjects( int color )
+void World::ResetCapturedObjects( const int color )
 {
     map_captureobj.ResetColor( color );
 }
 
-void World::ClearFog( int colors )
+void World::ClearFog( int colors ) const
 {
     colors = Players::GetPlayerFriends( colors );
 
@@ -921,7 +951,7 @@ const UltimateArtifact & World::GetUltimateArtifact() const
 
 bool World::DiggingForUltimateArtifact( const fheroes2::Point & center )
 {
-    Maps::Tiles & tile = GetTiles( center.x, center.y );
+    Maps::Tile & tile = getTile( center.x, center.y );
 
     // Get digging hole sprite.
     MP2::ObjectIcnType objectIcnType = MP2::OBJ_ICN_TYPE_UNKNOWN;
@@ -934,7 +964,7 @@ bool World::DiggingForUltimateArtifact( const fheroes2::Point & center )
         return false;
     }
 
-    tile.pushBottomLayerAddon( Maps::TilesAddon( Maps::BACKGROUND_LAYER, Maps::getNewObjectUID(), objectIcnType, imageIndex ) );
+    tile.pushGroundObjectPart( Maps::ObjectPart( Maps::BACKGROUND_LAYER, Maps::getNewObjectUID(), objectIcnType, imageIndex ) );
 
     if ( ultimate_artifact.isPosition( tile.GetIndex() ) && !ultimate_artifact.isFound() ) {
         ultimate_artifact.markAsFound();
@@ -974,7 +1004,7 @@ std::string World::DateString() const
 
 uint32_t World::CountObeliskOnMaps()
 {
-    const size_t res = std::count_if( vec_tiles.begin(), vec_tiles.end(), []( const Maps::Tiles & tile ) { return MP2::OBJ_OBELISK == tile.GetObject( false ); } );
+    const size_t res = std::count_if( vec_tiles.begin(), vec_tiles.end(), []( const Maps::Tile & tile ) { return MP2::OBJ_OBELISK == tile.getMainObjectType( false ); } );
     return res > 0 ? static_cast<uint32_t>( res ) : 6;
 }
 
@@ -985,10 +1015,10 @@ void World::ActionForMagellanMaps( int color )
 
     const int alliedColors = Players::GetPlayerFriends( color );
 
-    for ( Maps::Tiles & tile : vec_tiles ) {
+    for ( Maps::Tile & tile : vec_tiles ) {
         if ( tile.isWater() ) {
             if ( isAIPlayer && tile.isFog( color ) ) {
-                AI::Get().revealFog( tile, kingdom );
+                AI::Planner::Get().revealFog( tile, kingdom );
             }
 
             tile.ClearFog( alliedColors );
@@ -999,7 +1029,11 @@ void World::ActionForMagellanMaps( int color )
 MapEvent * World::GetMapEvent( const fheroes2::Point & pos )
 {
     std::list<MapObjectSimple *> res = map_objects.get( pos );
-    return !res.empty() ? static_cast<MapEvent *>( res.front() ) : nullptr;
+    if ( res.empty() ) {
+        return nullptr;
+    }
+
+    return dynamic_cast<MapEvent *>( res.front() );
 }
 
 MapObjectSimple * World::GetMapObject( uint32_t uid )
@@ -1287,23 +1321,27 @@ std::list<Route::Step> World::getPath( const Heroes & hero, int targetIndex )
 void World::resetPathfinder()
 {
     _pathfinder.reset();
-    AI::Get().resetPathfinder();
+    AI::Planner::Get().resetPathfinder();
 }
 
 void World::updatePassabilities()
 {
-    for ( Maps::Tiles & tile : vec_tiles ) {
-        tile.updateEmpty();
+    for ( Maps::Tile & tile : vec_tiles ) {
+        // If tile is empty then update tile's object type if needed.
+        if ( tile.getMainObjectType() == MP2::OBJ_NONE ) {
+            tile.updateObjectType();
+        }
+
         tile.setInitialPassability();
     }
 
     // Once the original passabilities are set we know all neighbours. Now we have to update passabilities based on neighbours.
-    for ( Maps::Tiles & tile : vec_tiles ) {
+    for ( Maps::Tile & tile : vec_tiles ) {
         tile.updatePassability();
     }
 }
 
-void World::PostLoad( const bool setTilePassabilities )
+void World::PostLoad( const bool setTilePassabilities, const bool updateUidCounterToMaximum )
 {
     if ( setTilePassabilities ) {
         updatePassabilities();
@@ -1313,18 +1351,60 @@ void World::PostLoad( const bool setTilePassabilities )
     _allTeleports.clear();
 
     for ( const int32_t index : Maps::GetObjectPositions( MP2::OBJ_STONE_LITHS ) ) {
-        _allTeleports[GetTiles( index ).GetObjectSpriteIndex()].push_back( index );
+        const auto * objectPart = Maps::getObjectPartByActionType( getTile( index ), MP2::OBJ_STONE_LITHS );
+        if ( objectPart == nullptr ) {
+            // It looks like it is a broken map. No way the tile doesn't have this object.
+            assert( 0 );
+            continue;
+        }
+
+        _allTeleports[objectPart->icnIndex].push_back( index );
     }
 
     // Cache all tiles that contain a certain part of the whirlpool (depending on object sprite index).
     _allWhirlpools.clear();
 
-    for ( const int32_t index : Maps::GetObjectPositions( MP2::OBJ_WHIRLPOOL ) ) {
-        _allWhirlpools[GetTiles( index ).GetObjectSpriteIndex()].push_back( index );
+    // Whirlpools are unique objects because they can have boats on them which are leftovers from heroes
+    // which disembarked on land. Tiles with boats and whirlpools are marked as Boat objects.
+    // So, searching by type is not accurate as these tiles will be skipped.
+    for ( const auto & [index, objectPart] : Maps::getObjectParts( MP2::OBJ_WHIRLPOOL ) ) {
+        assert( objectPart != nullptr );
+
+        _allWhirlpools[objectPart->icnIndex].push_back( index );
+    }
+
+    // Cache all positions of Eye of Magi objects.
+    _allEyeOfMagi.clear();
+    for ( const int32_t index : Maps::GetObjectPositions( MP2::OBJ_EYE_OF_MAGI ) ) {
+        _allEyeOfMagi.emplace_back( index );
     }
 
     resetPathfinder();
     ComputeStaticAnalysis();
+
+    // Find the maximum UID value.
+    uint32_t maxUid = 0;
+
+    for ( const Maps::Tile & tile : vec_tiles ) {
+        maxUid = std::max( tile.getMainObjectPart()._uid, maxUid );
+
+        for ( const auto & part : tile.getGroundObjectParts() ) {
+            maxUid = std::max( part._uid, maxUid );
+        }
+
+        for ( const auto & part : tile.getTopObjectParts() ) {
+            maxUid = std::max( part._uid, maxUid );
+        }
+    }
+
+    if ( updateUidCounterToMaximum ) {
+        // And set the UID counter value with the found maximum.
+        Maps::setLastObjectUID( maxUid );
+    }
+    else {
+        // Check that 'getNewObjectUID()' will return values that will not match the existing ones on the started map.
+        assert( Maps::getLastObjectUID() >= maxUid );
+    }
 }
 
 uint32_t World::GetMapSeed() const
@@ -1353,114 +1433,149 @@ bool World::isAnyKingdomVisited( const MP2::MapObjectType objectType, const int3
     return false;
 }
 
-StreamBase & operator<<( StreamBase & msg, const CapturedObject & obj )
+OStreamBase & operator<<( OStreamBase & stream, const CapturedObject & obj )
 {
-    return msg << obj.objcol << obj.guardians;
+    return stream << obj.objCol << obj.guardians;
 }
 
-StreamBase & operator>>( StreamBase & msg, CapturedObject & obj )
+IStreamBase & operator>>( IStreamBase & stream, CapturedObject & obj )
 {
-    return msg >> obj.objcol >> obj.guardians;
+    return stream >> obj.objCol >> obj.guardians;
 }
 
-StreamBase & operator<<( StreamBase & msg, const MapObjects & objs )
+OStreamBase & operator<<( OStreamBase & stream, const MapObjects & objs )
 {
-    msg << static_cast<uint32_t>( objs.size() );
-    for ( MapObjects::const_iterator it = objs.begin(); it != objs.end(); ++it )
-        if ( ( *it ).second ) {
-            const MapObjectSimple & obj = *( *it ).second;
-            msg << ( *it ).first << obj.GetType();
+    const std::map<uint32_t, std::unique_ptr<MapObjectSimple>> & objectsRef = objs._objects;
 
-            switch ( obj.GetType() ) {
-            case MP2::OBJ_EVENT:
-                msg << static_cast<const MapEvent &>( obj );
-                break;
+    stream.put32( static_cast<uint32_t>( objectsRef.size() ) );
 
-            case MP2::OBJ_SPHINX:
-                msg << static_cast<const MapSphinx &>( obj );
-                break;
+    for ( const auto & [uid, obj] : objectsRef ) {
+        assert( obj && obj->GetUID() == uid );
 
-            case MP2::OBJ_SIGN:
-                msg << static_cast<const MapSign &>( obj );
-                break;
+        if ( const auto * objPtr = dynamic_cast<const MapEvent *>( obj.get() ); objPtr != nullptr ) {
+            stream << uid << MP2::OBJ_EVENT << *objPtr;
+
+            continue;
+        }
+
+        if ( const auto * objPtr = dynamic_cast<const MapSphinx *>( obj.get() ); objPtr != nullptr ) {
+            stream << uid << MP2::OBJ_SPHINX << *objPtr;
+
+            continue;
+        }
+
+        if ( const auto * objPtr = dynamic_cast<const MapSign *>( obj.get() ); objPtr != nullptr ) {
+            stream << uid << MP2::OBJ_SIGN << *objPtr;
+
+            continue;
+        }
+
+        // Unknown object type
+        assert( 0 );
+    }
+
+    return stream;
+}
+
+IStreamBase & operator>>( IStreamBase & stream, MapObjects & objs )
+{
+    std::map<uint32_t, std::unique_ptr<MapObjectSimple>> & objectsRef = objs._objects;
+
+    const uint32_t size = stream.get32();
+
+    objectsRef.clear();
+
+    for ( uint32_t i = 0; i < size; ++i ) {
+        uint32_t uid{ 0 };
+        MP2::MapObjectType type{ MP2::OBJ_NONE };
+        stream >> uid;
+
+        static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_PRE2_1103_RELEASE, "Remove the logic below." );
+        if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_PRE2_1103_RELEASE ) {
+            int temp{ MP2::OBJ_NONE };
+            stream >> temp;
+
+            type = static_cast<MP2::MapObjectType>( temp );
+        }
+        else {
+            stream >> type;
+        }
+
+        std::unique_ptr<MapObjectSimple> obj = [&stream, type]() -> std::unique_ptr<MapObjectSimple> {
+            switch ( type ) {
+            case MP2::OBJ_EVENT: {
+                auto ptr = std::make_unique<MapEvent>();
+                stream >> *ptr;
+
+                return ptr;
+            }
+
+            case MP2::OBJ_SPHINX: {
+                auto ptr = std::make_unique<MapSphinx>();
+                stream >> *ptr;
+
+                return ptr;
+            }
+
+            case MP2::OBJ_SIGN: {
+                auto ptr = std::make_unique<MapSign>();
+                stream >> *ptr;
+
+                return ptr;
+            }
 
             default:
-                msg << obj;
                 break;
             }
+
+            return {};
+        }();
+
+        if ( !obj ) {
+            // Most likely the save file is corrupted.
+            stream.setFail();
+
+            continue;
         }
 
-    return msg;
-}
+        if ( obj->GetUID() != uid ) {
+            // Most likely the save file is corrupted.
+            stream.setFail();
 
-StreamBase & operator>>( StreamBase & msg, MapObjects & objs )
-{
-    uint32_t size = 0;
-    msg >> size;
-
-    objs.clear();
-
-    for ( uint32_t ii = 0; ii < size; ++ii ) {
-        int32_t index;
-        int type;
-        msg >> index >> type;
-
-        switch ( type ) {
-        case MP2::OBJ_EVENT: {
-            MapEvent * ptr = new MapEvent();
-            msg >> *ptr;
-            objs[index] = ptr;
-            break;
+            continue;
         }
 
-        case MP2::OBJ_SPHINX: {
-            MapSphinx * ptr = new MapSphinx();
-            msg >> *ptr;
-            objs[index] = ptr;
-            break;
-        }
-
-        case MP2::OBJ_SIGN: {
-            MapSign * ptr = new MapSign();
-            msg >> *ptr;
-            objs[index] = ptr;
-            break;
-        }
-
-        default: {
-            MapObjectSimple * ptr = new MapObjectSimple();
-            msg >> *ptr;
-            objs[index] = ptr;
-            break;
-        }
+        if ( const auto [dummy, inserted] = objectsRef.try_emplace( uid, std::move( obj ) ); !inserted ) {
+            // Most likely the save file is corrupted.
+            stream.setFail();
         }
     }
 
-    return msg;
+    return stream;
 }
 
-StreamBase & operator<<( StreamBase & msg, const World & w )
+OStreamBase & operator<<( OStreamBase & stream, const World & w )
 {
-    return msg << w.width << w.height << w.vec_tiles << w.vec_heroes << w.vec_castles << w.vec_kingdoms << w._customRumors << w.vec_eventsday << w.map_captureobj
-               << w.ultimate_artifact << w.day << w.week << w.month << w.heroIdAsWinCondition << w.heroIdAsLossCondition << w.map_objects << w._seed;
+    return stream << w.width << w.height << w.vec_tiles << w.vec_heroes << w.vec_castles << w.vec_kingdoms << w._customRumors << w.vec_eventsday << w.map_captureobj
+                  << w.ultimate_artifact << w.day << w.week << w.month << w.heroIdAsWinCondition << w.heroIdAsLossCondition << w.map_objects << w._seed;
 }
 
-StreamBase & operator>>( StreamBase & msg, World & w )
+IStreamBase & operator>>( IStreamBase & stream, World & w )
 {
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1010_RELEASE, "Remove the logic below." );
     if ( Game::GetVersionOfCurrentSaveFile() < FORMAT_VERSION_1010_RELEASE ) {
         uint16_t width = 0;
         uint16_t height = 0;
 
-        msg >> width >> height;
+        stream >> width >> height;
         w.width = width;
         w.height = height;
     }
     else {
-        msg >> w.width >> w.height;
+        stream >> w.width >> w.height;
     }
 
-    msg >> w.vec_tiles >> w.vec_heroes >> w.vec_castles >> w.vec_kingdoms >> w._customRumors >> w.vec_eventsday >> w.map_captureobj >> w.ultimate_artifact >> w.day
+    stream >> w.vec_tiles >> w.vec_heroes >> w.vec_castles >> w.vec_kingdoms >> w._customRumors >> w.vec_eventsday >> w.map_captureobj >> w.ultimate_artifact >> w.day
         >> w.week >> w.month >> w.heroIdAsWinCondition >> w.heroIdAsLossCondition;
 
     static_assert( LAST_SUPPORTED_FORMAT_VERSION < FORMAT_VERSION_1010_RELEASE, "Remove the logic below." );
@@ -1469,11 +1584,11 @@ StreamBase & operator>>( StreamBase & msg, World & w )
         ++w.heroIdAsLossCondition;
     }
 
-    msg >> w.map_objects >> w._seed;
+    stream >> w.map_objects >> w._seed;
 
-    w.PostLoad( false );
+    w.PostLoad( false, true );
 
-    return msg;
+    return stream;
 }
 
 void EventDate::LoadFromMP2( const std::vector<uint8_t> & data )
@@ -1549,7 +1664,7 @@ void EventDate::LoadFromMP2( const std::vector<uint8_t> & data )
     // - string
     //    Null terminated string containing the event text.
 
-    StreamBuf dataStream( data );
+    ROStreamBuf dataStream( data );
 
     dataStream.skip( 1 );
 
@@ -1601,7 +1716,7 @@ void EventDate::LoadFromMP2( const std::vector<uint8_t> & data )
         colors |= Color::PURPLE;
     }
 
-    message = dataStream.toString();
+    message = dataStream.getString();
 
     DEBUG_LOG( DBG_GAME, DBG_INFO, "A timed event which occurs at day " << firstOccurrenceDay << " contains a message: " << message )
 }
@@ -1630,12 +1745,12 @@ bool EventDate::isAllow( const int col, const uint32_t date ) const
     return ( ( date - firstOccurrenceDay ) % repeatPeriodInDays ) == 0;
 }
 
-StreamBase & operator<<( StreamBase & msg, const EventDate & obj )
+OStreamBase & operator<<( OStreamBase & stream, const EventDate & obj )
 {
-    return msg << obj.resource << obj.isApplicableForAIPlayers << obj.firstOccurrenceDay << obj.repeatPeriodInDays << obj.colors << obj.message << obj.title;
+    return stream << obj.resource << obj.isApplicableForAIPlayers << obj.firstOccurrenceDay << obj.repeatPeriodInDays << obj.colors << obj.message << obj.title;
 }
 
-StreamBase & operator>>( StreamBase & msg, EventDate & obj )
+IStreamBase & operator>>( IStreamBase & stream, EventDate & obj )
 {
-    return msg >> obj.resource >> obj.isApplicableForAIPlayers >> obj.firstOccurrenceDay >> obj.repeatPeriodInDays >> obj.colors >> obj.message >> obj.title;
+    return stream >> obj.resource >> obj.isApplicableForAIPlayers >> obj.firstOccurrenceDay >> obj.repeatPeriodInDays >> obj.colors >> obj.message >> obj.title;
 }
