@@ -42,6 +42,7 @@
 #include "castle.h"
 #include "color.h"
 #include "game_over.h"
+#include "game_static.h"
 #include "heroes.h"
 #include "kingdom.h"
 #include "logging.h"
@@ -62,6 +63,7 @@
 #include "resource.h"
 #include "serialize.h"
 #include "settings.h"
+#include "spell.h"
 #include "world.h" // IWYU pragma: associated
 #include "world_object_uid.h"
 
@@ -711,6 +713,8 @@ bool World::loadResurrectionMap( const std::string & filename )
     const auto & miscellaneousObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_MISCELLANEOUS );
     const auto & waterObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_WATER );
     const auto & artifactObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_ARTIFACTS );
+    const auto & treasuresObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_TREASURES );
+    const auto & powerUpsObjects = Maps::getObjectsByGroup( Maps::ObjectGroup::ADVENTURE_POWER_UPS );
 
 #if defined( WITH_DEBUG )
     std::set<uint32_t> standardMetadataUIDs;
@@ -719,8 +723,21 @@ bool World::loadResurrectionMap( const std::string & filename )
     std::set<uint32_t> sphinxMetadataUIDs;
     std::set<uint32_t> signMetadataUIDs;
     std::set<uint32_t> adventureMapEventMetadataUIDs;
+    std::set<uint32_t> selectionObjectMetadataUIDs;
 #endif
 
+    const auto areSpellsValid = []( const Maps::Map_Format::SelectionObjectMetadata & metadata, const int spellLevel ) {
+        if ( metadata.selectedItems.empty() ) {
+            // No spells are being set which means that we fall to the default behavior.
+            return false;
+        }
+
+        const std::vector<int32_t> & spells = Spell::getAllSpellIdsSuitableForSpellBook( spellLevel );
+
+        return std::all_of( metadata.selectedItems.begin(), metadata.selectedItems.end(),
+                            [&spells]( const int32_t spellId ) { return std::find( spells.begin(), spells.end(), spellId ) != spells.end(); } );
+    };
+
     std::set<size_t> hiredHeroTileId;
 
     for ( size_t tileId = 0; tileId < map.tiles.size(); ++tileId ) {
@@ -908,6 +925,21 @@ bool World::loadResurrectionMap( const std::string & filename )
                     }
                     break;
                 }
+                case MP2::OBJ_PYRAMID: {
+                    if ( map.selectionObjectMetadata.find( object.id ) == map.selectionObjectMetadata.end() ) {
+                        break;
+                    }
+#if defined( WITH_DEBUG )
+                    selectionObjectMetadataUIDs.emplace( object.id );
+#endif
+                    const auto & metadata = map.selectionObjectMetadata[object.id];
+
+                    if ( areSpellsValid( metadata, 5 ) ) {
+                        vec_tiles[static_cast<int32_t>( tileId )].metadata()[0] = Rand::Get( metadata.selectedItems );
+                    }
+
+                    break;
+                }
                 case MP2::OBJ_SIGN: {
 #if defined( WITH_DEBUG )
                     signMetadataUIDs.emplace( object.id );
@@ -965,6 +997,33 @@ bool World::loadResurrectionMap( const std::string & filename )
 
                     break;
                 }
+                case MP2::OBJ_WITCHS_HUT: {
+                    if ( map.selectionObjectMetadata.find( object.id ) == map.selectionObjectMetadata.end() ) {
+                        break;
+                    }
+#if defined( WITH_DEBUG )
+                    selectionObjectMetadataUIDs.emplace( object.id );
+#endif
+
+                    const auto areSkillsValid = []( const Maps::Map_Format::SelectionObjectMetadata & metadata ) {
+                        if ( metadata.selectedItems.empty() ) {
+                            // No skills are being set which means that we fall to the default behavior.
+                            return false;
+                        }
+
+                        const std::vector<int32_t> & skills = GameStatic::getSecondarySkillsForWitchsHut();
+
+                        return std::all_of( metadata.selectedItems.begin(), metadata.selectedItems.end(),
+                                            [&skills]( const int32_t skillId ) { return std::find( skills.begin(), skills.end(), skillId ) != skills.end(); } );
+                    };
+
+                    const auto & metadata = map.selectionObjectMetadata[object.id];
+                    if ( areSkillsValid( metadata ) ) {
+                        vec_tiles[static_cast<int32_t>( tileId )].metadata()[0] = Rand::Get( metadata.selectedItems );
+                    }
+
+                    break;
+                }
                 default:
                     // Other objects do not have metadata as of now.
                     break;
@@ -1012,6 +1071,66 @@ bool World::loadResurrectionMap( const std::string & filename )
                     tileData[0] = tileData[0] - 1U;
                 }
             }
+            else if ( object.group == Maps::ObjectGroup::ADVENTURE_TREASURES ) {
+                assert( object.index < treasuresObjects.size() );
+
+                const auto & objectInfo = treasuresObjects[object.index];
+
+                if ( objectInfo.objectType == MP2::OBJ_RESOURCE && map.standardMetadata.find( object.id ) != map.standardMetadata.end() ) {
+                    // Some maps may have resource objects being set by older Editor versions.
+                    // Therefore, we cannot have a strict check whether metadata for this object exists like we do for other objects.
+#if defined( WITH_DEBUG )
+                    standardMetadataUIDs.emplace( object.id );
+#endif
+
+                    std::array<uint32_t, 3> & tileData = vec_tiles[static_cast<int32_t>( tileId )].metadata();
+
+                    tileData[0] = objectInfo.metadata[0];
+                    tileData[1] = map.standardMetadata[object.id].metadata[0];
+                }
+            }
+            else if ( object.group == Maps::ObjectGroup::ADVENTURE_POWER_UPS ) {
+                assert( object.index < powerUpsObjects.size() );
+
+                const MP2::MapObjectType objectType = powerUpsObjects[object.index].objectType;
+
+                switch ( objectType ) {
+                case MP2::OBJ_SHRINE_FIRST_CIRCLE:
+                case MP2::OBJ_SHRINE_SECOND_CIRCLE:
+                case MP2::OBJ_SHRINE_THIRD_CIRCLE: {
+                    if ( map.selectionObjectMetadata.find( object.id ) == map.selectionObjectMetadata.end() ) {
+                        break;
+                    }
+#if defined( WITH_DEBUG )
+                    selectionObjectMetadataUIDs.emplace( object.id );
+#endif
+                    const auto & metadata = map.selectionObjectMetadata[object.id];
+
+                    int spellLevel = 0;
+                    if ( objectType == MP2::OBJ_SHRINE_FIRST_CIRCLE ) {
+                        spellLevel = 1;
+                    }
+                    else if ( objectType == MP2::OBJ_SHRINE_SECOND_CIRCLE ) {
+                        spellLevel = 2;
+                    }
+                    else if ( objectType == MP2::OBJ_SHRINE_THIRD_CIRCLE ) {
+                        spellLevel = 3;
+                    }
+                    else {
+                        assert( 0 );
+                        spellLevel = 1;
+                    }
+
+                    if ( areSpellsValid( metadata, spellLevel ) ) {
+                        vec_tiles[static_cast<int32_t>( tileId )].metadata()[0] = Rand::Get( metadata.selectedItems );
+                    }
+
+                    break;
+                }
+                default:
+                    break;
+                }
+            }
         }
     }
 
@@ -1022,6 +1141,7 @@ bool World::loadResurrectionMap( const std::string & filename )
     assert( sphinxMetadataUIDs.size() == map.sphinxMetadata.size() );
     assert( signMetadataUIDs.size() == map.signMetadata.size() );
     assert( adventureMapEventMetadataUIDs.size() == map.adventureMapEventMetadata.size() );
+    assert( selectionObjectMetadataUIDs.size() == map.selectionObjectMetadata.size() );
 
     for ( const uint32_t uid : standardMetadataUIDs ) {
         assert( map.standardMetadata.find( uid ) != map.standardMetadata.end() );
@@ -1046,6 +1166,10 @@ bool World::loadResurrectionMap( const std::string & filename )
     for ( const uint32_t uid : adventureMapEventMetadataUIDs ) {
         assert( map.adventureMapEventMetadata.find( uid ) != map.adventureMapEventMetadata.end() );
     }
+
+    for ( const uint32_t uid : selectionObjectMetadataUIDs ) {
+        assert( map.selectionObjectMetadata.find( uid ) != map.selectionObjectMetadata.end() );
+    }
 #endif
 
     // Load daily events.
